unit udmCustomers;

interface

uses
  SysUtils
  , Classes
  , DB
  , DBTables
  , udmBDEBase
  ;

type
  TdmCustomers = class(TdmBDEBase)
    Customer: TQuery;
    CustomerCustNo: TFloatField;
    CustomerCompany: TStringField;
    CustomerAddr1: TStringField;
    CustomerAddr2: TStringField;
    CustomerCity: TStringField;
    CustomerState: TStringField;
    CustomerZip: TStringField;
    CustomerCountry: TStringField;
    CustomerPhone: TStringField;
    CustomerFAX: TStringField;
    CustomerTaxRate: TFloatField;
    CustomerContact: TStringField;
    CustomerLastInvoiceDate: TDateTimeField;
    Orders: TQuery;
    OrdersOrderNo: TFloatField;
    OrdersCustNo: TFloatField;
    OrdersSaleDate: TDateTimeField;
    OrdersShipDate: TDateTimeField;
    OrdersEmpNo: TIntegerField;
    OrdersShipToContact: TStringField;
    OrdersShipToAddr1: TStringField;
    OrdersShipToAddr2: TStringField;
    OrdersShipToCity: TStringField;
    OrdersShipToState: TStringField;
    OrdersShipToZip: TStringField;
    OrdersShipToCountry: TStringField;
    OrdersShipToPhone: TStringField;
    OrdersShipVIA: TStringField;
    OrdersPO: TStringField;
    OrdersTerms: TStringField;
    OrdersPaymentMethod: TStringField;
    OrdersItemsTotal: TCurrencyField;
    OrdersTaxRate: TFloatField;
    OrdersFreight: TCurrencyField;
    OrdersAmountPaid: TCurrencyField;
    Items: TQuery;
    ItemsOrderNo: TFloatField;
    ItemsItemNo: TFloatField;
    ItemsPartNo: TFloatField;
    ItemsQty: TIntegerField;
    ItemsDiscount: TFloatField;
    Parts: TQuery;
    PartsPartNo: TFloatField;
    PartsVendorNo: TFloatField;
    PartsDescription: TStringField;
    PartsOnHand: TFloatField;
    PartsOnOrder: TFloatField;
    PartsCost: TCurrencyField;
    PartsListPrice: TCurrencyField;
    Vendors: TQuery;
    VendorsVendorNo: TFloatField;
    VendorsVendorName: TStringField;
    VendorsAddress1: TStringField;
    VendorsAddress2: TStringField;
    VendorsCity: TStringField;
    VendorsState: TStringField;
    VendorsZip: TStringField;
    VendorsCountry: TStringField;
    VendorsPhone: TStringField;
    VendorsFAX: TStringField;
    VendorsPreferred: TBooleanField;
    procedure CustomerAfterScroll(DataSet: TDataSet);
    procedure OrdersAfterScroll(DataSet: TDataSet);
    procedure ItemsAfterScroll(DataSet: TDataSet);
    procedure CustomerNewRecord(DataSet: TDataSet);
    procedure OrdersNewRecord(DataSet: TDataSet);
    procedure ItemsNewRecord(DataSet: TDataSet);
    procedure CustomerBeforeDelete(DataSet: TDataSet);
    procedure OrdersBeforeDelete(DataSet: TDataSet);
    procedure OrdersSaleDateValidate(Sender: TField);
    procedure OrdersShipDateValidate(Sender: TField);
    procedure ItemsPartNoChange(Sender: TField);
    procedure PartsAfterOpen(DataSet: TDataSet);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

{var
  dmCustomers: TdmCustomers;}

implementation

uses
  uDMConnection
  , Dialogs
  ;

{$R *.dfm}

procedure TdmCustomers.CustomerAfterScroll(DataSet: TDataSet);
begin
  // Get the orders for this customer
  if Orders.Active then
    Orders.Close;
  Orders.ParamByName('CustNo').AsFloat := CustomerCustNo.AsFloat;
end;

procedure TdmCustomers.OrdersAfterScroll(DataSet: TDataSet);
begin
  // Get the items for this order
  if Items.Active then
    Items.Close;
  Items.ParamByName('OrderNo').AsFloat := OrdersOrderNo.AsFloat;
end;

procedure TdmCustomers.ItemsAfterScroll(DataSet: TDataSet);
begin
  // Get the part for this item
  if Parts.Active then
    Parts.Close;
  Parts.ParamByName('PartNo').AsFloat := ItemsPartNo.AsFloat;
  Parts.Open;
end;

procedure TdmCustomers.CustomerNewRecord(DataSet: TDataSet);
begin
  // Initialise a new Customer record
  Customer.FieldByName('CustNo').AsInteger := gDMConnection.NextCust;
end;

procedure TdmCustomers.OrdersNewRecord(DataSet: TDataSet);
begin
  // Initialise a new Orders record
  Orders.FieldByName('OrderNo').AsInteger := gDMConnection.NextOrder;
  Orders.FieldByName('CustNo').AsInteger :=
                                      Customer.FieldByName('CustNo').AsInteger;
  Orders.FieldByName('SaleDate').AsDateTime := Date;
end;

procedure TdmCustomers.ItemsNewRecord(DataSet: TDataSet);
begin
  // Initialise a new Items record
  Items.FieldByName('OrderNo').AsInteger :=
                                      Orders.FieldByName('OrderNo').AsInteger;
  Items.FieldByName('ItemNo').AsInteger :=
              gDMConnection.NextItem(Orders.FieldByName('OrderNo').AsInteger);
end;

procedure TdmCustomers.CustomerBeforeDelete(DataSet: TDataSet);
begin
  // Delete any associated Orders records
  gDMConnection.DBConnection.StartTransaction;
  try
    if not Orders.Active then
      Orders.Open;
    if not Orders.IsEmpty then
    begin
      Orders.Last;
      while not Orders.IsEmpty do
      begin
        Orders.Delete;
      end;
    end;
    Orders.Close;
    gDMConnection.DBConnection.Commit;
  except
    gDMConnection.DBConnection.Rollback;
  end;
end;

procedure TdmCustomers.OrdersBeforeDelete(DataSet: TDataSet);
var
  bInTrans: boolean;
begin
  // Delete any associated Items records
  bInTrans := gDMConnection.DBConnection.InTransaction;
  if not bInTrans then
    gDMConnection.DBConnection.StartTransaction;
  try
    if not Items.Active then
      Items.Open;
    if not Items.IsEmpty then
    begin
      Items.Last;
      while not Items.IsEmpty do
      begin
        Items.Delete;
      end;
    end;
    Items.Close;
    if not bInTrans then
      gDMConnection.DBConnection.Commit;
  except
    if not bInTrans then
      gDMConnection.DBConnection.Rollback;
  end;
end;

procedure TdmCustomers.OrdersSaleDateValidate(Sender: TField);
begin
  if (Sender.AsDateTime < Date) then
    raise Exception.Create('The Sale Date cannot be before today!');
end;

procedure TdmCustomers.OrdersShipDateValidate(Sender: TField);
begin
  if (Sender.AsDateTime < Date) then
    raise Exception.Create('The Ship Date cannot be before today!');
  if (Sender.AsDateTime < OrdersSaleDate.AsDateTime) then
    raise Exception.Create('The Ship Date cannot be before the Sale Date!');
end;

procedure TdmCustomers.ItemsPartNoChange(Sender: TField);
begin
  // Get the part for this item
  if Parts.Active then
    Parts.Close;
  Parts.ParamByName('PartNo').AsFloat := ItemsPartNo.AsFloat;
  Parts.Open;
end;

procedure TdmCustomers.PartsAfterOpen(DataSet: TDataSet);
begin
  // Get the vendor for this part
  if Vendors.Active then
    Vendors.Close;
  Vendors.ParamByName('VendorNo').AsInteger := PartsVendorNo.AsInteger;
  Vendors.Open;
end;

end.
